# Generated by Django 4.2.7 on 2023-11-12 13:06

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="BuildProcess",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("machine", models.CharField(db_index=True, max_length=255)),
                ("build_id", models.CharField(max_length=255)),
                ("build_host", models.CharField(max_length=255)),
                ("package", models.CharField(max_length=255)),
                ("phase", models.CharField(max_length=255)),
                ("start_time", models.DateTimeField()),
            ],
            options={
                "unique_together": {("machine", "build_id", "build_host", "package")}
            },
        )
    ]
