type BuildProcess {
  id: String!
  machine: String!
  buildHost: String!
  package: String!
  phase: String!
  startTime: DateTime
}

input BuildProcessInput {
  id: String!
  machine: String!
  buildHost: String!
  package: String!
  phase: String!
  startTime: DateTime
}

extend type Query {
  buildProcesses(includeFinal: Boolean): [BuildProcess!]!
}

extend type Mutation {
  addBuildProcess(process: BuildProcessInput!): Error
}
